# This module handles selecting units.
# Also a bunch of other cursor-related stuff

import thing, gamestate, data, lawn
import pygame, math, random


# A graphic that shows a round region to be selected
# Caches previous calls
class Selector:
  alpha = 40
  images = {}
  def getimage(self, size):
    if size in Selector.images: return Selector.images[size]
    s = pygame.Surface((size,size/2)).convert_alpha()
    s.fill((0,0,0,0))
    if size > 3:
      pygame.draw.ellipse(s, (255, 255, 255, Selector.alpha), s.get_rect())
      pygame.draw.ellipse(s, (255, 255, 255, 255), s.get_rect(), 1)
    Selector.images[size] = s
    return s
  def draw(self, surf, p0, p1):
    x0, y0 = p0
    x1, y1 = p1
    d = int(math.sqrt((x0-x1)**2 + (y0-y1)**2 * 4.))
    i = self.getimage(d)
    rect = i.get_rect()
    rect.center = int((x0+x1)/2), int((y0+y1)/2.)
    surf.blit(i, rect)

# the state of a mouse
class Mstate:
  shiftmask = pygame.KMOD_LSHIFT | pygame.KMOD_RSHIFT
  def __init__(self):
    self.b1, self.b2, self.b3 = pygame.mouse.get_pressed()
    self.px, self.py = self.p = pygame.mouse.get_pos()
    self.mods = pygame.key.get_mods()
    self.shift = self.mods & Mstate.shiftmask
    self.x, self.y = gamestate.Ptxy(self.p)
  # a one-pixel sprite at the mouse's position, used for collision detection
  def sprite(self):
    s = pygame.sprite.Sprite()
    s.rect = pygame.Rect(self.px, self.py, 1, 1)
    return s

rarrow = larrow = rrect = lrect = None

def startup():
  global rarrow, larrow, rrect, lrect, mlast
  rarrow = data.loadimg("turnarrow.png").convert_alpha()
  larrow = pygame.transform.flip(rarrow, True, False)
  rrect = rarrow.get_rect()
  lrect = larrow.get_rect()
  rrect.bottomleft = 30, gamestate.sy - 20
  lrect.bottomright = gamestate.sx - 30, gamestate.sy - 20
  mlast = Mstate()

def closest(things):
  y, r = -100000, None
  for t in things:
    layer = thing.Thing.group.get_layer_of_sprite(t)
    if layer > y: y, r = layer, t
  return r

class MenuBlock(pygame.sprite.Sprite):
  group = pygame.sprite.Group()
  dict = {}
  width, height = 48, 48
  color0 = 140, 70, 0
  color1 = 100, 50, 0
  color2 = 60, 30, 0
  def __init__(self, name, text, icon=None):
    pygame.sprite.Sprite.__init__(self)
    self.width, self.height = 48, 48
    self.image0 = pygame.Surface((self.width, self.height))
    self.image0.fill(self.color1)
    for d in (0,1):
      w, h = self.width-1-d, self.height-1-d
      pygame.draw.line(self.image0, self.color0, (d,d), (d,h))
      pygame.draw.line(self.image0, self.color0, (d,d), (w,d))
      pygame.draw.line(self.image0, self.color2, (w,h), (d,h))
      pygame.draw.line(self.image0, self.color2, (w,h), (w,d))
    self.rect = self.image0.get_rect()
    self.name = name
    self.text = text
    y = self.rect.bottom - 4
    for n in reversed(text):
      s = f.render(n, True, (192, 192, 192))
      r = s.get_rect()
      r.midbottom = self.rect.centerx, y
      self.image0.blit(s, r)
      y -= 10
    self.icon = icon
    if icon:
      r = icon.get_rect()
      r.center = self.image0.get_rect().centerx, y/2
      self.image0.blit(icon, r)
    self.image = self.image0
    self.group.add(self)
    self.dict[name] = self
    self.x, self.y = 0, -self.height
    self.fx, self.fy = self.x, self.y
    self.wait = 0
    self.active = False
    self.setpos()
  def diminish(self, n, k = 4, m = 10):
    if n < 0: return -self.diminish(-n)
    if n == 0: return 0
    if n < k: return 1
    if n/k > m: return m
    return n/k
  def setpos(self):
    self.rect.right = gamestate.screen.get_rect().right + self.x
    self.rect.top = gamestate.screen.get_rect().top + self.y
  def update(self):
    if self.wait:
      self.wait -= 1
      return
    if self.fx != self.x or self.fy != self.y:
      self.x += self.diminish(self.fx - self.x)
      self.y += self.diminish(self.fy - self.y)
      self.setpos()


class Cursor():
  mblox = []
  def __init__(self):
    self.mlast = None  # mouse state on last call
    self.mdown = None  # mouse state last time the button was pushed
    self.mstate = None
    self.tomove = []
    self.dragging = False
    self.pclass = None
    self.pname = ""
    self.coverageframe = False
    self.toupdatemenu = True
  def drawselector(self):
    if self.mstate.b1 and self.mlast.b1 and self.gonefar():
      Selector().draw(gamestate.screen, self.mdown.p, self.mstate.p)
  def makecoverage(self):
    self.coverageframe = pygame.Surface(gamestate.lawndim)
    self.coverageframe.fill((0,0,0))
    self.coverageframe.set_colorkey((0,0,0))
    for t in thing.StarPlant.group:
      px, py = gamestate.Pxy(t.x, t.y)
      sx, sy = t.rstar * gamestate.kx, t.rstar * gamestate.ky
      r = pygame.Rect(int(px - sx), int(py - sy - gamestate.lawny0), int(2*sx), int(2*sy))
      pygame.draw.ellipse(self.coverageframe, (255, 255, 0), r)
    lawn.cookiecut(self.coverageframe)
    self.coverageframe.set_alpha(60)
  def drawcoverage(self, condition=True):
    if condition and self.pclass != thing.StarPlant: return
    if not self.coverageframe: self.makecoverage()
    r = self.coverageframe.get_rect()
    r.midbottom = gamestate.screen.get_rect().midbottom
    gamestate.screen.blit(self.coverageframe, r)
  def canbuild(self):
    if not self.pclass: return False
    return thing.canbuild((self.mstate.x, self.mstate.y), self.pclass)
  def drawcursor(self):
    if self.pclass:
      i = self.pclass.imagebank.image(("routline", "outline")[self.canbuild()])
      rect = i.get_rect()
      rect.topleft = self.mstate.px - self.pclass.dpx, self.mstate.py - self.pclass.dpy
      gamestate.screen.blit(i, rect)
  def click(self):
    if self.mstate.py <= MenuBlock.height:
      self.menuclick()
      return
    if self.pclass and self.canbuild():
      s, d = thing.nearestthing((self.mstate.x, self.mstate.y), thing.Acorn)
      if s:
        s.dropeverything()
        s.move((self.mstate.x, self.mstate.y))
        s.doq.append("self.become%s()" % self.pname)
    if not self.pclass:
      hits = pygame.sprite.spritecollide(self.mstate.sprite(), thing.Thing.sgroup, False)
      if len(hits):  # on a unit
        if not self.mstate.shift: thing.unselectall()
        closest(hits).select()
      else:  # on nothing
        if self.mstate.x**2 + self.mstate.y**2 < 1.:
          for i in thing.Thing.igroup:
            self.tomove.append((i,self.mstate.x,self.mstate.y))
  def menuclick(self):
    for m in pygame.sprite.spritecollide(self.mstate.sprite(), MenuBlock.group, False):
      if m.name == "quit":
        gamestate.done = True
      elif m.name == "select":
        for s in thing.Thing.sgroup: s.select()
      elif m.name == "unselect":
        thing.unselectall()
        self.pclass = None
      elif m.name == "starplant":
        if thing.Thing.igroup:
          for t in thing.Thing.igroup: t.becomestarplant()
        else:
          self.pclass, self.pname = thing.StarPlant, "starplant"
      elif m.name == "nabdrop":
        for s in thing.Thing.igroup:
          if not s.gots: s.nabnearestdrop()
      elif m.name == "usedrop":
        for s in thing.Thing.igroup:
          if s.gots:
            t, d = thing.nearestthing((s.x, s.y), thing.Structure)
            s.use(t)
      elif m.name == "dropper":
        self.pclass, self.pname = thing.Dewdropper, "dewdropper"
      elif m.name == "fertilize":
        if thing.Thing.igroup:
          for t in thing.Thing.igroup: t.fertilize()
        
  def rightclick(self):
    self.pclass = None
    self.pname = ""
    thing.unselectall()
  def dragup(self):
    cx, cy = (self.mstate.x + self.mdown.x) / 2, (self.mstate.y + self.mdown.y) / 2
    r2 = (self.mstate.x - cx)**2 + (self.mstate.y - cy)**2
    hits = [s for s in thing.Thing.sgroup if (s.x-cx)**2+(s.y-cy)**2 <= r2]
    if len(hits) and not self.mstate.shift: thing.unselectall()
    for i in hits: i.select()
  def gonefar(self):
    dx, dy = self.mstate.px - self.mdown.px, self.mstate.py - self.mdown.py
    return abs(dx) >= 4 or abs(dy) >= 4
  def update(self):
    self.mstate = Mstate()
    if not self.mlast: self.mlast = self.mstate
    if len(self.tomove):
      i, tx, ty = self.tomove.pop(0)
      i.move((tx, ty))
    if self.mstate.b1:
      if rrect.collidepoint(self.mstate.p): gamestate.addtheta(2)
      if lrect.collidepoint(self.mstate.p): gamestate.addtheta(-2)
    if self.mlast.b1 and not self.mstate.b1:  # mouseup
      if not self.gonefar():  # click
        self.click()
      else:
        self.dragup()
    if self.mstate.b1 and not self.mlast.b1:  # mousedown
      self.mdown = self.mstate
    if self.mstate.b3 and not self.mstate.b1:  # right click
      self.rightclick()
    self.mlast = self.mstate
    self.setmenu("unselect", bool(len(thing.Thing.igroup) or self.pclass))
    self.setmenu("nabdrop", bool(len(thing.Thing.igroup) and len(thing.Dewdrop.group)))
    self.setmenu("usedrop", thing.Thing.igroup and max([len(t.gots) for t in thing.Thing.igroup]))
    self.setmenu("starplant", self.pclass != thing.StarPlant)
    self.setmenu("dropper", self.pclass != thing.Dewdropper)
    if self.toupdatemenu: self.updatemenu()
    self.toupdatemenu = False
  def drawinfo(self):
    hits = pygame.sprite.spritecollide(self.mstate.sprite(), thing.Thing.tgroup, False)
    pat = closest(hits)
    s = f.render("Pointing at: %s" % pat, True, (192, 192, 192))
    gamestate.screen.blit(s, (5,5))
    s = f.render("Selected: %d Acorns" % len(thing.Thing.igroup.sprites()), True, (192, 192, 192))
    gamestate.screen.blit(s, (5,20))
    s = f.render("Coverage: %d%%" % int(gamestate.coverage * 100), True, (192, 192, 192))
    gamestate.screen.blit(s, (5,35))
    s = f.render("Acorns: %d/%d" % (len(thing.Acorn.group), gamestate.maxacorns), True, (192, 192, 192))
    gamestate.screen.blit(s, (5,50))

    gamestate.screen.blit(rarrow, rrect)
    gamestate.screen.blit(larrow, lrect)
  def makemenu(self):
    acornimg = data.loadimg("acorn-mini.png")
    ximg = data.loadimg("x-mini.png")
    spimg = data.loadimg("starplant-mini.png")
    ddimg = data.loadimg("dewdropper-mini.png")
    ddpimg = data.loadimg("dewdrop-mini.png")
    self.mblox.append(MenuBlock("quit", ("Pause/","Quit"), ximg))
    self.mblox.append(MenuBlock("select", ("Select","All"), acornimg))
    self.mblox.append(MenuBlock("unselect", ("Unselect",), ximg))
    self.mblox.append(MenuBlock("starplant", ("Build", "StarPlant"), spimg))
    self.mblox.append(MenuBlock("dropper", ("Build", "Dropper"), ddimg))
    self.mblox.append(MenuBlock("nabdrop", ("Get", "Dew Drop"), ddpimg))
    self.mblox.append(MenuBlock("usedrop", ("Use", "Dew Drop"), ddpimg))
    self.mblox.append(MenuBlock("fertilize", ("Fertilize",)))
    for m in self.mblox:
      m.x, m.y = m.fx, m.fy = -1, -48
      m.setpos()
      m.active = True
  def setmenu(self, mname, active):
    m = MenuBlock.dict[mname]
    if m.active == active: return
    m.active = active
    self.toupdatemenu = True
  def updatemenu(self):
    nactive = 0
    # Step 1: scroll up newly inactive blocks
    p = 0
    for m in self.mblox:
      if not m.active and m.y >= 0:
        m.fy = -48
        m.wait = p
        p += 3
    # Step 2: scroll left/right still active blocks
    p += 5
    nactive = 0
    p2 = p + 5
    for m in self.mblox:
      if m.active:
        if m.y >= 0:
          m.fx = -nactive * MenuBlock.width
          if m.x != m.fx: p += 3
          m.wait = p
        else:
          m.x = m.fx = -nactive * MenuBlock.width
          m.fy = 0
          m.wait = p2
          p2 += 3
        nactive += 1
  def randomizemenus(self):
    for m in self.mblox:
      m.active = random.randint(0, 2)
    self.updatemenu()
  def drawmenu(self):
    MenuBlock.group.update()
    MenuBlock.group.draw(gamestate.screen)    

pygame.font.init()
f = pygame.font.Font(None, 14)

