# This module implements a set of related images for a sprite.
# An image is retrieved by passing a string to the image() method.
# The requests are cached, for faster lookup times on subsequent calls
# Requests must follow the naming convention:
#   empty string: basic image
#   ghost: image at 50% transparency
#   ghostNNN: ghost except for the bottom NNN pixels
#   stony: image converted to grayscale
#   underNNN: scooted down NNN pixels
#   outline: black-and-white ghostly outline
#   routline: black-and-red outline

import data
import pygame

class Imagebank:
  def __init__(self, f0, width = None):
    self.dict = {}
    self.f0 = f0
    self.width = width

  def image0(self):
    i = data.loadimg(self.f0 + ".png").convert_alpha()
    if self.width == None: self.width = i.get_width()
    height = self.width * i.get_height() / i.get_width()
    self.image0 = pygame.transform.smoothscale(i, (self.width, height))
    self.dict[""] = self.image0
    return self.image0
  def image(self, name = ""):
    if name in self.dict: return self.dict[name]
    if name == "":
      self.dict[name] = self.image0()
    elif name == "ghost":
      self.dict[name] = self.ghost()
    elif name == "stony":
      self.dict[name] = self.stony()
    elif name == "outline":
      self.dict[name] = self.outline()
    elif name == "routline":
      self.dict[name] = self.routline()
    elif "ghost" in name:
      self.dict[name] = self.ghost(int(name[5:]))
    elif "under" in name:
      self.dict[name] = self.under(int(name[5:]))
    elif "rglow" in name:
      self.dict[name] = self.glow(int(name[5:])/100., (255, 128, 128))
    elif "glow" in name:
      self.dict[name] = self.glow(int(name[4:])/100.)
    else:
      return None
    return self.dict[name]
  def ghost(self, n = 0, f = 0.5):
    w, h = self.image0.get_size()
    i = pygame.Surface((w, h)).convert_alpha()
    for y in range(h):
      for x in range(w):
        r,g,b,a = self.image0.get_at((x,y))
        if h-y > n: a = int(f*a)
        i.set_at((x,y), (r,g,b,a))
    return i
  def under(self, n):
    i0 = self.image()
    i = pygame.Surface(i0.get_size()).convert_alpha()
    i.fill((0,0,0,0))
    r = i0.get_rect()
    r.top = n
    i.blit(i0, r)
    return i
  def stony(self):
    w, h = self.image0.get_size()
    i = pygame.Surface((w, h)).convert_alpha()
    for y in range(h):
      for x in range(w):
        r,g,b,a = self.image0.get_at((x,y))
        v = int((r+g+b)/6)
        i.set_at((x,y), (v,v,v,a))
    return i
  def glow(self, f, c0=(255, 255, 255)):
    r0, g0, b0 = c0
    mr = [int(j*(1-f) + r0*f) for j in range(256)]
    mg = [int(j*(1-f) + g0*f) for j in range(256)]
    mb = [int(j*(1-f) + b0*f) for j in range(256)]
    w, h = self.image0.get_size()
    i = pygame.Surface((w, h)).convert_alpha()
    for y in range(h):
      for x in range(w):
        r,g,b,a = self.image0.get_at((x,y))
        i.set_at((x,y), (mr[r],mg[g],mb[b],a))
    return i
  def outline(self):
    i0 = self.image()
    w, h = i0.get_size()
    i = pygame.Surface((w, h)).convert_alpha()
    for y in range(h):
      for x in range(w):
        r,g,b,a = i0.get_at((x,y))
        if a < 128:
          i.set_at((x,y), (0,0,0,0))
        else:
          i.set_at((x,y), (255, 255, 255, 128))
    for y in range(h):
      for x in range(w):
        r,g,b,a0 = i.get_at((x,y))
        if y < h-1:
          r,g,b,a1 = i.get_at((x,y+1))
          if a0 and not a1: i.set_at((x,y), (0,0,0,255))
          if a1 and not a0: i.set_at((x,y+1), (0,0,0,255))
        if x < w-1:
          r,g,b,a1 = i.get_at((x+1,y))
          if a0 and not a1: i.set_at((x,y), (0,0,0,255))
          if a1 and not a0: i.set_at((x+1,y), (0,0,0,255))
    return i
  def routline(self, color = (255, 0, 0, 128)):
    i0 = self.image("outline")
    w, h = i0.get_size()
    i = pygame.Surface((w, h)).convert_alpha()
    for y in range(h):
      for x in range(w):
        r,g,b,a = i0.get_at((x,y))
        if g:
          i.set_at((x,y), color)
        else:
          i.set_at((x,y), (r,g,b,a))
    return i


