# This module creates the texture frames for the rotating stage

import gamestate
import math, random, pygame
from pygame.locals import *

# create a surface with a gaussian alpha
def gaussian(color, s = 50, amax = 40, nsig = 3):
  h = pygame.Surface((s,s)).convert_alpha()
  h.fill(color)
  # fast lookup on the exponential
  a = [int(math.exp(-d/200.)*amax) for d in range(nsig**2 * 220)]
  sig = s / 2. / nsig
  for x in range(s):
    for y in range(s):
      d = ((x - s/2.)**2 + (y - s/2.)**2) / (sig**2)
      alpha = a[int(d*100)]
      h.set_at((x,y), color + (alpha,))
  return h

# clip an outer ellipse
def cookiecut(surf, color = (0,0,0), color2 = (255,255,255)):
  c = pygame.Surface(surf.get_size())
  c.set_colorkey(color2)
  pygame.draw.ellipse(c, color2, c.get_rect())
  surf.blit(c, (0, 0))

# make the un-transformed overhead view
def overhead(d, color1, color2):
  b = pygame.Surface((d, d))
  b.fill(color1)
  h = gaussian(color2)
  for j in range(200):
    s = random.randint(200, 400)
    x = random.randint(-s, d+s)
    y = random.randint(-s, d+s)
    surf = pygame.transform.scale(h, (s, s))
    b.blit(surf, (x, y))
  return b

lawn0 = None
# creates the frames that shall serve as the stage
def makelawn(color1, color2):
  global lawn0
  lawn0 = overhead(gamestate.lawnx, color1, color2)

frames = {}
def frame(theta):
  if theta in frames: return frames[theta]
  b3 = pygame.Surface((gamestate.lawnx, gamestate.lawnx))
  b4 = pygame.Surface(gamestate.lawndim)
  b2 = pygame.transform.rotate(lawn0, theta)
  r = b2.get_rect()
  r.center = b3.get_rect().center
  b3.blit(b2, r)
  pygame.transform.scale(b3, gamestate.lawndim, b4)
  frames[theta] = pygame.Surface(gamestate.lawndim)
  frames[theta].blit(b4, (0, 0))
  frames[theta].set_colorkey((0,0,0))
  cookiecut(frames[theta])
  return frames[theta]


